import base64
import builtins
import datetime
import io
import json
import pickle
from importlib import import_module


class RestrictedUnpickler(pickle.Unpickler):
    modules = {}

    def find_class(self, module, name):
        if module == "builtins":
            if name in ("bool", "int", "float", "str", "list", "dict"):
                return getattr(builtins, name)

        if module == "datetime":
            if name in ("date", "datetime"):
                return getattr(datetime, name)

        if module in RestrictedUnpickler.modules and name in RestrictedUnpickler.modules[module]:
            return getattr(import_module(module), name)

        raise pickle.UnpicklingError(f"Unpickling object of type {module}.{name} forbidden.")


class Format:
    BINARY = 0x00
    BASE64 = 0x01
    PICKLE = 0x02
    JSON = 0x03

    @staticmethod
    def from_string(format):
        if format == "binary":
            return Format.BINARY

        if format == "base64":
            return Format.BASE64

        if format == "pickle":
            return Format.PICKLE

        if format == "json":
            return Format.JSON

        raise ValueError("Invalid format string.")

    @staticmethod
    def encode(format, data):
        if format == Format.BINARY:
            return data

        if format == Format.BASE64:
            return base64.b64encode(data)

        if format == Format.PICKLE:
            return base64.b64encode(Format.pickle_data(data))

        if format == Format.JSON:
            return json.dumps(data).encode()

        raise Exception("Unreachable code.")

    @staticmethod
    def decode(format, data):
        if format == Format.BINARY:
            return data

        if format == Format.BASE64:
            return base64.b64decode(data)

        if format == Format.PICKLE:
            return Format.unpickle_data(base64.b64decode(data))

        if format == Format.JSON:
            return json.loads(data.decode())

        raise Exception("Unreachable code.")

    @staticmethod
    def pickle_data(data):
        return pickle.dumps(data)

    @staticmethod
    def unpickle_data(data):
        return RestrictedUnpickler(io.BytesIO(data)).load()
