import contextlib
import contextvars
import ssl


@contextlib.asynccontextmanager
async def asynchronous_context():
    yield contextvars.copy_context()


def create_tls_context(*, server, configuration):
    if server:
        context = ssl.SSLContext(ssl.PROTOCOL_TLS)
        context.load_cert_chain(configuration["server_certificate"], keyfile=configuration["server_key"])
    else:
        context = ssl.SSLContext(ssl.PROTOCOL_TLS)
        context.load_verify_locations(cafile="certificates/ca.crt")
        context.verify_mode = ssl.VerifyMode.CERT_REQUIRED
        context.check_hostname = True

    context.set_ciphers(":".join(configuration["ciphers"]))

    context.options |= ssl.OP_NO_TLSv1
    context.options |= ssl.OP_NO_TLSv1_1
    context.options |= ssl.OP_NO_TLSv1_2
    context.options |= ssl.OP_SINGLE_DH_USE
    context.options |= ssl.OP_SINGLE_ECDH_USE

    return context
