from .errors import ProtocolError
from .format import Format


class ReaderWrapper:
    def __init__(self, reader):
        self.reader = reader

    async def read_byte(self):
        value = await self.reader.read(1)
        return int.from_bytes(value, byteorder="big")

    async def read_integer(self):
        value = await self.reader.read(4)
        return int.from_bytes(value, byteorder="big")

    async def read_bytes(self, format, length_max):
        length = await self.read_integer()

        if length > length_max:
            raise ProtocolError("Data length exceed.")

        data = await self.reader.read(length)

        if isinstance(format, bool):
            return data.decode()

        return Format.decode(format, data)


class WriterWrapper:
    def __init__(self, writer):
        self.writer = writer

    def write_byte(self, value):
        value = value.to_bytes(1, byteorder="big")
        self.writer.write(value)

    def write_integer(self, value):
        value = value.to_bytes(4, byteorder="big")
        self.writer.write(value)

    def write_bytes(self, format, data):
        if isinstance(format, bool):
            data = data.encode()
        else:
            data = Format.encode(format, data)

        self.write_integer(len(data))
        self.writer.write(data)

    async def drain(self):
        await self.writer.drain()

    def close(self):
        return self.writer.close()
